@@ -1,4 +1,3 @@
-##' Fits a log-normal, gamma, or Weibull model to doubly interval
 ##' censored survival data
 ##'
 ##' \code{dic.fit} fits a parametric accelerated failure time model to survival
@@ -27,8 +26,8 @@
 ##'   exposure (E) and symptom onset (S). Also, a "type" column must be
 ##'   specified and have entries with 0, 1, or 2, corresponding to doubly
 ##'   interval-censored, single interval-censored or exact observations,
-##'   respsectively.
-##' @param start.par2 starting value for 2nd parameter of desired distribtution
+##'   respectively.
+##' @param start.par2 starting value for 2nd parameter of desired distribution
 ##' @param opt.method method used by optim
 ##' @param par1.int the log-scale interval of possible median values (in the
 ##'   same units as the observations in dat).  Narrowing this interval can help
@@ -49,7 +48,7 @@
 ##' dic.fit(fluA.inc.per, dist="L")
 ##' @references Reich NG et al.  Statistics in Medicine.  Estimating incubation
 ##'   periods with coarse data. 2009.
-##'   \url{http://www3.interscience.wiley.com/journal/122507367/abstract}
+##'   \url{https://www.ncbi.nlm.nih.gov/pubmed/19598148}
 
 dic.fit <- function(dat,
 		    start.par2=log(2),
@@ -298,10 +297,16 @@ fw1 <- function(t, EL, ER, SL, SR, par1, par2, dist){
     ## function that calculates the first function for the DIC integral
     if (dist=="W"){
         (ER-SL+t) * dweibull(x=t,shape=par1,scale=par2)
+    } else if (dist=="off1W") {
+      (ER-SL+t) * dweibullOff1(x=t,shape=par1,scale=par2)
     } else if (dist=="G") {
         (ER-SL+t) * dgamma(x=t, shape=par1, scale=par2)
+    } else if (dist=="off1G") {
+      (ER-SL+t) * dgammaOff1(x=t, shape=par1, scale=par2)
     } else if (dist =="L"){
         (ER-SL+t) * dlnorm(x=t, meanlog=par1, sdlog=par2)
+    } else if (dist =="off1L"){
+      (ER-SL+t) * dlnormOff1(x=t, meanlog=par1, sdlog=par2)
     } else {
         stop("distribution not supported")
     }
@@ -311,11 +316,17 @@ fw1 <- function(t, EL, ER, SL, SR, par1, par2, dist){
 fw3 <- function(t, EL, ER, SL, SR, par1, par2, dist){
     ## function that calculates the third function for the DIC integral
     if (dist == "W"){
-	(SR-EL-t) * dweibull(x=t, shape=par1, scale=par2)
-    } else if (dist == "G"){
+    	(SR-EL-t) * dweibull(x=t, shape=par1, scale=par2)
+    } else if (dist == "off1W"){
+      (SR-EL-t) * dweibullOff1(x=t, shape=par1, scale=par2)
+    }  else if (dist == "G"){
     	(SR-EL-t) * dgamma(x=t, shape=par1, scale=par2)
+    }  else if (dist == "off1G"){
+      (SR-EL-t) * dgammaOff1(x=t, shape=par1, scale=par2)
     } else if (dist == "L") {
         (SR-EL-t) * dlnorm(x=t, meanlog=par1, sdlog=par2)
+    } else if (dist == "off1L"){
+      (SR-EL-t) * dlnormOff1(x=t, meanlog=par1, sdlog=par2)
     } else {
         stop("distribution not supported")
     }
@@ -333,7 +344,7 @@ lik <- function(par1, par2, EL, ER, SL, SR, type, dist){
 
 ## calculates the DIC likelihood by integration
 diclik <- function(par1, par2, EL, ER, SL, SR, dist){
-
+  
     ## if symptom window is bigger than exposure window
     if(SR-SL>ER-EL){
         dic1 <- integrate(fw1, lower=SL-ER, upper=SL-EL,
@@ -344,12 +355,21 @@ diclik <- function(par1, par2, EL, ER, SL, SR, dist){
         if (dist == "W"){
             dic2 <- (ER-EL)*
                 (pweibull(SR-ER, shape=par1, scale=par2) - pweibull(SL-EL, shape=par1, scale=par2))
+        } else if (dist == "off1W"){
+          dic2 <- (ER-EL)*
+            (pweibullOff1(SR-ER, shape=par1, scale=par2) - pweibullOff1(SL-EL, shape=par1, scale=par2))
         } else if (dist == "G"){
             dic2 <- (ER-EL)*
                 (pgamma(SR-ER, shape=par1, scale=par2) - pgamma(SL-EL, shape=par1, scale=par2))
+        } else if (dist == "off1G"){
+          dic2 <- (ER-EL)*
+            (pgammaOff1(SR-ER, shape=par1, scale=par2) - pgammaOff1(SL-EL, shape=par1, scale=par2))
         } else if (dist == "L") {
             dic2 <- (ER-EL)*
                 (plnorm(SR-ER, par1, par2) - plnorm(SL-EL, par1, par2))
+        } else if (dist == "off1L") {
+          dic2 <- (ER-EL)*
+            (plnormOff1(SR-ER, par1, par2) - plnormOff1(SL-EL, par1, par2))
         } else {
             stop("distribution not supported")
         }
@@ -370,12 +390,21 @@ diclik <- function(par1, par2, EL, ER, SL, SR, dist){
         if (dist == "W"){
             dic2 <- (SR-SL)*
                 (pweibull(SL-EL, shape=par1, scale=par2) - pweibull(SR-ER, shape=par1, scale=par2))
+        } else if (dist == "off1W"){
+          dic2 <- (SR-SL)*
+            (pweibullOff1(SL-EL, shape=par1, scale=par2) - pweibullOff1(SR-ER, shape=par1, scale=par2))
         } else if (dist == "G"){
             dic2 <- (SR-SL)*
                 (pgamma(SL-EL, shape=par1, scale=par2) - pgamma(SR-ER, shape=par1, scale=par2))
+        } else if (dist == "off1G"){
+          dic2 <- (SR-SL)*
+            (pgammaOff1(SL-EL, shape=par1, scale=par2) - pgammaOff1(SR-ER, shape=par1, scale=par2))
         } else if (dist == "L"){
             dic2 <- (SR-SL)*
                 (plnorm(SL-EL, par1, par2) - plnorm(SR-ER, par1, par2))
+        } else if (dist == "off1L"){
+          dic2 <- (SR-SL)*
+            (plnormOff1(SL-EL, par1, par2) - plnormOff1(SR-ER, par1, par2))
         } else {
             stop("distribution not supported")
         }
@@ -391,8 +420,10 @@ diclik <- function(par1, par2, EL, ER, SL, SR, dist){
 ## this dic likelihood is designed for data that has overlapping intervals
 diclik2 <- function(par1, par2, EL, ER, SL, SR, dist){
     if(SL>ER) {
+
         return(diclik(par1, par2, EL, ER, SL, SR, dist))
     } else {
+
         lik1 <- integrate(diclik2.helper1, lower=EL, upper=SL,
                           SL=SL, SR=SR, par1=par1, par2=par2, dist=dist)$value
         lik2 <- integrate(diclik2.helper2, lower=SL, upper=ER,
@@ -405,10 +436,16 @@ diclik2 <- function(par1, par2, EL, ER, SL, SR, dist){
 diclik2.helper1 <- function(x, SL, SR, par1, par2, dist){
     if (dist =="W"){
         pweibull(SR-x, shape=par1, scale=par2) - pweibull(SL-x, shape=par1, scale=par2)
+    } else if (dist =="off1W") {
+      pweibullOff1(SR-x, shape=par1, scale=par2) - pweibullOff1(SL-x, shape=par1, scale=par2)
     } else if (dist =="G") {
         pgamma(SR-x, shape=par1, scale=par2) - pgamma(SL-x, shape=par1, scale=par2)
+    } else if (dist=="off1G"){
+       pgammaOff1(SR-x, shape=par1, scale=par2) - pgammaOff1(SL-x, shape=par1, scale=par2)
     } else if (dist == "L"){
         plnorm(SR-x, par1, par2) - plnorm(SL-x, par1, par2)
+    } else if (dist == "off1L"){
+      plnormOff1(SR-x, par1, par2) - plnormOff1(SL-x, par1, par2)
     } else {
      stop("distribution not supported")     
     }
@@ -417,10 +454,16 @@ diclik2.helper1 <- function(x, SL, SR, par1, par2, dist){
 diclik2.helper2 <- function(x, SR, par1, par2, dist){
     if (dist =="W"){
         pweibull(SR-x, shape=par1, scale=par2)
+    } else if (dist =="off1W") {
+      pweibullOff1(SR-x, shape=par1, scale=par2)
     } else if (dist =="G") {
         pgamma(SR-x, shape=par1, scale=par2)
+    } else if (dist =="off1G") {
+       pgammaOff1(SR-x, shape=par1, scale=par2)
     } else if (dist=="L"){
 	       plnorm(SR-x, par1, par2)
+    } else if (dist=="off1L"){
+      plnormOff1(SR-x, par1, par2)
     } else {
         stop("distribution not supported")     
     }
@@ -431,10 +474,16 @@ siclik <- function(par1, par2, EL, ER, SL, SR, dist){
     ## calculates the SIC likelihood as the difference in CDFs
     if (dist =="W"){
         pweibull(SR-EL, shape=par1, scale=par2) - pweibull(SL-ER, shape=par1, scale=par2)
+    } else if (dist=="off1W") {
+      pweibullOff1(SR-EL, shape=par1, scale=par2) - pweibullOff1(SL-ER, shape=par1, scale=par2)
+    } else if (dist=="off1G") {
+       pgammaOff1(SR-EL, shape=par1, scale=par2) - pgammaOff1(SL-ER, shape=par1, scale=par2)
     } else if (dist =="G") {
         pgamma(SR-EL, shape=par1, scale=par2) - pgamma(SL-ER, shape=par1, scale=par2)
     } else if (dist == "L"){
         plnorm(SR-EL, par1, par2) - plnorm(SL-ER, par1, par2)
+    } else if (dist=="off1L") {
+      plnormOff1(SR-EL, par1, par2) - plnormOff1(SL-ER, par1, par2)
     } else {
        stop("distribution not supported")
     }
@@ -447,10 +496,16 @@ exactlik <- function(par1, par2, EL, ER, SL, SR, dist){
     ##     so it doesn't matter which pair we use in the forpar1la below.
     if (dist =="W"){
         dweibull(SR-EL, shape=par1, scale=par2)
+    } else if (dist=="off1W") {
+      dweibullOff1(SR-EL, shape=par1, scale=par2)
+    } else if (dist=="off1G") {
+        dgammaOff1(SR-EL, shape=par1, scale=par2)
     } else if (dist =="G") {
         dgamma(SR-EL, shape=par1, scale=par2)
     } else if (dist == "L") {
         dlnorm(SR-EL, par1, par2)
+    } else if (dist == "off1L") {
+      dlnormOff1(SR-EL, par1, par2)
     } else {
         stop("distribution not supported")     
     }
@@ -479,7 +534,7 @@ loglikhd <- function(pars, dat, dist) {
       
   
     #if the distribution is erlanf transform correctly for gamma
-    if (dist=="E") {return(loglikhd(c(log(pars[1]),pars[2]),dat,dist="G"))}
+    if(dist %in% c("E")) {return(loglikhd(c(log(pars[1]),pars[2]),dat,dist="G"))}
     
     ## calculates the log-likelihood of DIC data
     ## dat must have EL, ER, SL, SR and type columns  
@@ -493,6 +548,7 @@ loglikhd <- function(pars, dat, dist) {
     ## cat(sprintf("par1 = %.2f, par2 = %.2f \n",par1, par2))  ## for debugging
     n <- nrow(dat)
     totlik <- 0
+    
     for(i in 1:n){
       #cat(i,"start\n") ##DEBUG
         totlik <- totlik + log(lik(par1, par2, type=dat[i,"type"],
@@ -599,14 +655,14 @@ single.boot <- function(par1.s,par2.s,opt.method,dat.tmp,dist,...){
 ## Transforms parameters of a specific distriution for unbounded optimization
 ## returns vector of transformed parameters
 dist.optim.transform <- function(dist,pars){
-    if (dist == "G"){
+    if (dist == "G" || dist == "off1G"){
         return(log(pars)) # for shape and scale
-    } else if (dist == "W"){
+    } else if (dist == "W" || dist == "off1W"){
         return(log(pars)) # for shape and scale
     } else if (dist == "E"){
         #shape not transformed, logged
         return(c(pars[1],log(pars[2])))
-    } else if (dist == "L"){
+    } else if (dist == "L" || dist == "off1L"){
         return(c(pars[1],log(pars[2]))) # for meanlog, sdlog
     } else {
         stop(sprintf("Distribtion (%s) not supported",dist))
@@ -616,14 +672,14 @@ dist.optim.transform <- function(dist,pars){
 ## Untransforms parameters before entering likelihood
 ## returns vector of untransformed parameters
 dist.optim.untransform <- function(dist,pars){
-    if (dist == "G"){
+    if (dist == "G" || dist=="off1G"){
         return(exp(pars)) # for shape and scale
-    } else if (dist == "W"){
+    } else if (dist == "W" || dist == "off1W"){
         return(exp(pars)) # for shape and scale
     } else if (dist == "E"){
         #shape identity, scale logged in estimation scale        
         return(c(pars[1],exp(pars[2])))
-    } else if (dist == "L"){
+    } else if (dist == "L" || dist == "off1L"){
         return(c(pars[1],exp(pars[2]))) # for meanlog, sdlog
     } else {
         stop(sprintf("Distribtion (%s) not supported",dist))
@@ -647,3 +703,79 @@ check.data.structure <- function(dat){
     return(NULL)
 }
 
+
+##' Function that calculates pgamma with a offset of 1 (i.e., 1 is equivalent to 0)
+##' 
+##' @param x value to calculate pgamma at
+##' @param replace0 should we replace 0 with epsilon
+##' @param ... other parameters to pgamma
+##'
+##' @return pgamma offset
+##' 
+pgammaOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- pgamma(x-1, ...)
+  if (replace0 && sum(rc<=0)>0) {
+  
+    rc[which(rc<=0)] <- 10^-8
+  }
+  return(rc)
+}
+
+plnormOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- plnorm(x-1, ...)
+  if (replace0 && sum(rc<=0)>0) {
+    
+    rc[which(rc<=0)] <- 10^-8
+  }
+  return(rc)
+}
+
+pweibullOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- pweibull(x-1, ...)
+  if (replace0 && sum(rc<=0)>0) {
+    
+    rc[which(rc<=0)] <- 10^-8
+  }
+  return(rc)
+}
+
+
+
+
+##' Function that calculates dgamma with a offset of 1 (i.e., 1 is equivalent to 0)
+##' 
+##' @param x value to calculate dgamma at
+##' @param replace0 should we replace 0 with epsilon
+##' @param ... other parameters to dgamma
+##'
+##' @return dgamma offset
+##' 
+dgammaOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- dgamma(x-1, ...)
+
+  if (replace0 && rc<=0) {
+
+    rc <- 10^-8
+  }
+  return(rc)
+}
+
+dlnormOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- dlnorm(x-1, ...)
+  
+  if (replace0 && rc<=0) {
+    
+    rc <- 10^-8
+  }
+  return(rc)
+}
+
+dweibullOff1 <- function(x, replace0 = FALSE, ...) {
+  rc <- dweibull(x-1, ...)
+  
+  if (replace0 && rc<=0) {
+    
+    rc <- 10^-8
+  }
+  return(rc)
+}
\ No newline at end of file
